@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Author: Don Welch
+# Author: Don Welch, Naga Samrat Chowdary Narla
 #
 
 # Std Lib
@@ -27,8 +27,10 @@ import os.path
 import time
 import urllib # TODO: Replace with urllib2 (urllib is deprecated in Python 3.0)
 import StringIO
+import cStringIO
 import httplib
 import struct
+import string
 
 # Local
 from g import *
@@ -36,6 +38,7 @@ from codes import *
 import utils
 import status
 import pml
+import status
 from prnt import pcl, ldl, cups
 import models, mdns, slp
 from strings import StringTable
@@ -91,6 +94,11 @@ ip_pat = re.compile(r"""\b(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25
 dev_pat = re.compile(r"""/dev/.+""", re.IGNORECASE)
 usb_pat = re.compile(r"""(\d+):(\d+)""", re.IGNORECASE)
 
+### **********Lambda Function UniStar for checking type of arguments to constructor of class event*******************************
+
+UniStr = lambda title: isinstance(title, str) and utils.xrstrip(title, '\x00')[:128] or utils.xrstrip(title, '\x00')[:128].encode('utf-8')  
+
+
 #
 # Event Wrapper Class for pipe IPC
 #
@@ -99,13 +107,13 @@ class Event(object):
     def __init__(self, device_uri, printer_name, event_code,
                  username=prop.username, job_id=0, title='',
                  timedate=0):
-
-        self.device_uri = unicode(utils.xrstrip(device_uri, '\x00'))[:128].encode('utf-8')
-        self.printer_name = unicode(utils.xrstrip(printer_name, '\x00'))[:128].encode('utf-8')
+       # UniStr = lambda title: isinstance(title, str) and utils.xrstrip(title, '\x00')[:128] or utils.xrstrip(title, '\x00')[:128].encode('utf-8')
+        self.device_uri = UniStr(device_uri)
+        self.printer_name = UniStr(printer_name)
         self.event_code = int(event_code)
-        self.username = unicode(utils.xrstrip(username, '\x00'))[:32].encode('utf-8')
+        self.username = UniStr(username)
         self.job_id = int(job_id)
-        self.title = unicode(utils.xrstrip(title, '\x00'))[:128].encode('utf-8')
+        self.title = UniStr(title)
 
         if timedate:
             self.timedate = float(timedate)
@@ -1293,6 +1301,12 @@ class Device(object):
     def openEWS(self):
         return self.__openChannel(hpmudext.HPMUD_S_EWS_CHANNEL)
 
+    def openEWS_LEDM(self):
+        return self.__openChannel(hpmudext.HPMUD_S_EWS_LEDM_CHANNEL)
+    
+    def openLEDM(self):
+        return self.__openChannel(hpmudext.HPMUD_S_LEDM_SCAN)
+
     def closePrint(self):
         return self.__closeChannel(hpmudext.HPMUD_S_PRINT_CHANNEL)
 
@@ -1314,6 +1328,12 @@ class Device(object):
     def closeEWS(self):
         return self.__closeChannel(hpmudext.HPMUD_S_EWS_CHANNEL)
 
+    def closeEWS_LEDM(self):
+        return self.__closeChannel(hpmudext.HPMUD_S_EWS_LEDM_CHANNEL)
+    
+    def closeLEDM(self):
+        return self.__closeChannel(hpmudext.HPMUD_S_LEDM_SCAN)
+
     def openCfgUpload(self):
         return self.__openChannel(hpmudext.HPMUD_S_CONFIG_UPLOAD_CHANNEL)
 
@@ -2099,6 +2119,12 @@ class Device(object):
     def readEWS(self, bytes_to_read, stream=None, timeout=prop.read_timeout, allow_short_read=True):
         return self.__readChannel(self.openEWS, bytes_to_read, stream, timeout, allow_short_read)
 
+    def readEWS_LEDM(self, bytes_to_read, stream=None, timeout=prop.read_timeout, allow_short_read=True):
+        return self.__readChannel(self.openEWS_LEDM, bytes_to_read, stream, timeout, allow_short_read)
+
+    def readLEDM(self, bytes_to_read, stream=None, timeout=prop.read_timeout, allow_short_read=True):
+        return self.__readChannel(self.openLEDM, bytes_to_read, stream, timeout, allow_short_read)
+
     def readSoapFax(self, bytes_to_read, stream=None, timeout=prop.read_timeout, allow_short_read=True):
         return self.__readChannel(self.openSoapFax, bytes_to_read, stream, timeout, allow_short_read)
 
@@ -2179,6 +2205,12 @@ class Device(object):
     def writeEWS(self, data):
         return self.__writeChannel(self.openEWS, data)
 
+    def writeEWS_LEDM(self, data):
+        return self.__writeChannel(self.openEWS_LEDM, data)
+
+    def writeLEDM(self, data):
+        return self.__writeChannel(self.openLEDM, data)
+
     def writeCfgDownload(self, data):
         return self.__writeChannel(self.openCfgDownload, data)
 
@@ -2395,8 +2427,6 @@ class Device(object):
         self.hist = result
         return result
 
-
-
     def getEWSUrl(self, url, stream):
         try:
             if self.is_local:
@@ -2427,7 +2457,58 @@ class Device(object):
         finally:
             self.closeEWS()
 
+    def getEWSUrl_LEDM(self, url, stream, footer=""):
+        try:
+            url2 = "%s&loc=%s" % (self.device_uri.replace('hpfax:', 'hp:'), url)
+            data = self
+            opener = LocalOpenerEWS_LEDM({})
+            try:
+                if footer:
+                    return opener.open_hp(url2, data, footer)
+                else:
+                    return opener.open_hp(url2, data) 
+            except Error:
+                log.debug("Status read failed: %s" % url2)
+        finally:
+            self.closeEWS_LEDM()
+    
+    def getUrl_LEDM(self, url, stream, footer=""):
+        try:
+            url2 = "%s&loc=%s" % (self.device_uri.replace('hpfax:', 'hp:'), url)
+            data = self
+            opener = LocalOpener_LEDM({})
+            try:
+                if footer:
+                    return opener.open_hp(url2, data, footer)
+                else:
+                    return opener.open_hp(url2, data) 
+            except Error:
+                log.debug("Status read failed: %s" % url2)
 
+        finally:
+            self.closeLEDM()
+
+    def FetchLEDMUrl(self, url, footer=""):
+        data_fp = cStringIO.StringIO()
+        if footer:
+            data = self.getUrl_LEDM(url, data_fp, footer)
+        else:
+            data = self.getUrl_LEDM(url, data_fp)
+        if data:
+            data = data.split('\r\n\r\n', 1)[1]
+            if data:
+                data = status.clean(data)
+        return data
+ 
+    def readAttributeFromXml(self,uri,attribute):
+        stream = cStringIO.StringIO()
+        data = self.FetchLEDMUrl(uri)        
+        if not data:
+            log.error("Unable To read the XML data from device")
+            return ""
+        xmlDict = utils.XMLToDictParser().parseXML(data)  
+        return str(xmlDict[attribute])
+		
     def downloadFirmware(self, usb_bus_id=None, usb_device_id=None): # Note: IDs not currently used
         ok = False
         filename = os.path.join(prop.data_dir, "firmware", self.model.lower() + '.fw.gz')
@@ -2504,4 +2585,57 @@ class LocalOpener(urllib.URLopener):
         else:
             return response.fp
 
+# URLs: hp:/usb/HP_OfficeJet_7500?serial=00XXXXXXXXXX&loc=/hp/device/info_device_status.xml
+class LocalOpenerEWS_LEDM(urllib.URLopener):
+    def open_hp(self, url, dev, foot=""):
+        log.debug("open_hp(%s)" % url)
 
+        match_obj = http_pat_url.search(url)
+        loc = url.split("=")[url.count("=")]
+
+        dev.openEWS_LEDM()
+        if foot:
+            if "PUT" in foot:
+                dev.writeEWS_LEDM("""%s""" % foot)
+            else:
+                dev.writeEWS_LEDM("""POST %s HTTP/1.1\r\nContent-Type:text/xml\r\nContent-Length:%s\r\nAccept-Encoding: UTF-8\r\nHost:localhost\r\nUser-Agent:hplip\r\n\r\n """ % (loc, len(foot)))
+                dev.writeEWS_LEDM("""%s""" % foot)
+        else:
+            dev.writeEWS_LEDM("""GET %s HTTP/1.1\r\nAccept: text/plain\r\nHost:localhost\r\nUser-Agent:hplip\r\n\r\n""" % loc)
+
+        reply = xStringIO()
+
+        while dev.readEWS_LEDM(8080, reply, timeout=3):
+            pass
+
+        reply.seek(0)
+        return reply.getvalue()
+
+# URLs: hp:/usb/HP_OfficeJet_7500?serial=00XXXXXXXXXX&loc=/hp/device/info_device_status.xml
+class LocalOpener_LEDM(urllib.URLopener):
+    def open_hp(self, url, dev, foot=""):
+        log.debug("open_hp(%s)" % url)
+
+        match_obj = http_pat_url.search(url)
+        loc = url.split("=")[url.count("=")]
+
+        dev.openLEDM()
+        if foot:
+            if "PUT" in foot:
+                dev.writeLEDM("""%s""" % foot)
+            else:
+                dev.writeLEDM("""POST %s HTTP/1.1\r\nContent-Type:text/xml\r\nContent-Length:%s\r\nAccept-Encoding: UTF-8\r\nHost:localhost\r\nUser-Agent:hplip\r\n\r\n """ % (loc, len(foot)))
+                dev.writeLEDM("""%s""" % foot)
+        else:
+            dev.writeLEDM("""GET %s HTTP/1.1\r\nAccept: text/plain\r\nHost:localhost\r\nUser-Agent:hplip\r\n\r\n""" % loc)
+
+        reply = xStringIO()
+
+        while dev.readLEDM(8080, reply, timeout=3):
+            pass
+
+        reply.seek(0)
+        return reply.getvalue()
+
+
+    
