@@ -888,7 +888,7 @@ magic = [
     [0L, 'long', '=', 270L, 'unknown readable demand paged pure executable'],
     [0L, 'string', '=', 'begin uuencoded', 'or xxencoded text'],
     [0L, 'string', '=', 'xbtoa Begin', "btoa'd text"],
-    [0L, 'string', '=', '$012ship', "ship'd binary text"],
+    [0L, 'string', '=', '$\012ship', "ship'd binary text"],
     [0L, 'string', '=', 'Decode the following with bdeco', 'bencoded News text'],
     [11L, 'string', '=', 'must be converted with BinHex', 'BinHex binary text'],
     [0L, 'short', '=', 610L, 'Perkin-Elmer executable'],
@@ -1235,6 +1235,10 @@ def whatis(data):
         if ord(c) == 0:
             return 'data'
 
+    # its ASCII, now do C/CPP tests
+    if data.find('#include', 0, 256) > -1 or data.find('/***', 0, 256) > -1:
+        return 'text/cpp'
+
     # its ASCII, now do text tests
     if data.find('!/usr/local/bin/python2.7', 0, 256) > -1 or data.find('def ', 0, 8192) > -1:
         return 'application/x-python'
