@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Author: Stan Dolson
+# Author: Stan Dolson , Goutam Kodu
 #
 
 # Std Lib
@@ -462,9 +462,9 @@ def run_plugin_command(required=True, plugin_reason=PLUGIN_REASON_NONE):
     if need_sudo:
         su_sudo = utils.su_sudo()
     if su_sudo is "su":
-        name,version = utils.os_release()
-        log.debug("name = %s version = %s" %(name,version))
-        if ( name == 'Fedora' and version >= '14'):
+        name,version,is_su = utils.os_release()
+        log.debug("name = %s version = %s is_su = %s" %(name,version,is_su))
+        if ( name == 'Fedora' and version >= '14' and is_su == True):
            #using su opening GUI apps fail in Fedora 14. 
            #To run GUI apps as root, you need a root login shell (su -) in Fedora 14   
            su_sudo = 'su - -c "%s"'
@@ -479,16 +479,25 @@ def run_plugin_command(required=True, plugin_reason=PLUGIN_REASON_NONE):
     if not required:
         req = '--optional'
 
+
     if utils.which("hp-plugin"):
-        cmd = su_sudo % ("hp-plugin -u %s --reason %s" % (req, plugin_reason))
+        p_path="hp-plugin"
     else:
-        cmd = su_sudo % ("/usr/local/bin/python2.7 ./plugin.py -u %s --reason %s" % (req, plugin_reason))
+        p_path="/usr/local/bin/python2.7 ./plugin.py"
+
+    if 'gksu' in su_sudo:
+        cmd = su_sudo % ("%s -u %s --reason %s" % (p_path, req, plugin_reason))
+        cmd +=" -m" 
+        cmd += (" \"hp-plugin:- HP Device requires to install HP proprietary plugin. Please enter user (sudo) password to continue\"")
+    else:
+        cmd = su_sudo % ("%s -u %s --reason %s To_install_plugin_for_HP_Device" % (p_path, req, plugin_reason))
 
     log.debug("%s" % cmd)
     if password_f is not None:
         status, output = utils.run(cmd, log_output=True, password_func=password_f, timeout=1)
     else:
         status, output = utils.run(cmd, log_output=True, password_func=None, timeout=1)
+    
     return (status == 0, True)
 
 
