@@ -83,6 +83,7 @@ PLUGIN_INSTALL_ERROR_UNABLE_TO_RECV_KEYS = 7
 
 PING_TARGET = "www.google.com"
 HTTP_GET_TARGET = "http://www.google.com"
+PLUGIN_FALLBACK_LOCATION = 'http://hplipopensource.com/hplip-web/plugin/'
 
 EXPECT_WORD_LIST = [
     pexpect.EOF, # 0
@@ -250,6 +251,7 @@ class CoreInstall(object):
             'add_user_to_group': TYPE_STRING,
             'open_mdns_port' : TYPE_LIST, # command to use to open mdns multicast port 5353
             'acl_rules' : TYPE_BOOL, # Use ACL uDEV rules (Ubuntu 9.10+)
+            'libdir_path' : TYPE_STRING,
         }
 
         # components
@@ -1186,8 +1188,11 @@ class CoreInstall(object):
 
         if self.ppd_dir is not None:
             configure_cmd += ' --with-hpppddir=%s' % self.ppd_dir
-
-        if self.bitness == 64:
+            
+        libdir_path = self.get_distro_ver_data('libdir_path',False)
+        if libdir_path and self.bitness == 64:
+            configure_cmd += ' --libdir=%s' % (libdir_path)
+        elif self.bitness == 64:
             configure_cmd += ' --libdir=/usr/lib64'
 
         configure_cmd += ' --prefix=%s' % self.install_location
@@ -1795,7 +1800,12 @@ class CoreInstall(object):
                 return True
 
             if callback is not None:
-                callback("", "Password incorrect. %d attempt(s) left." % (3-x))
+                if "not in the sudoers file" in output:
+                    callback("", "%s is not in the sudoers file. Check privileges\n" %(os.getenv('USER')) )
+                    return False
+                else:    
+                    callback("", "Password incorrect. %d attempt(s) left." % (3-x ))
+
 
             x += 1
 
@@ -1930,7 +1940,21 @@ class CoreInstall(object):
 
         plugin_file = os.path.join(self.plugin_path, self.plugin_name)
 
+
+        #Check whether plugin is accessible in Openprinting.org website otherwise dowload plugin from alternate location.
+        wget = utils.which("wget")
+        if wget:
+            wget = os.path.join(wget, "wget")
+            cmd = "%s %s" % (wget, url)
+            log.debug(cmd)
+            status, output = self.run(cmd)
+            log.debug("wget returned: %d" % status)
+
         try:
+            if (status != 0) and 'file://' not in url:
+                url = os.path.join(PLUGIN_FALLBACK_LOCATION, self.plugin_name)
+                log.info("Plugin is not accessible. Trying to download it from fallback location: [%s]" % url)
+
             filename, headers = urllib.urlretrieve(url, plugin_file, callback)
         except IOError, e:
             log.error("Plug-in download failed: %s" % e.strerror)
@@ -1953,7 +1977,7 @@ class CoreInstall(object):
         try:
             filename, headers = urllib.urlretrieve(digsig_url, digsig_file, callback)
         except IOError, e:
-            log.error("Plug-in GPG file download failed: %s" % e.strerror)
+            log.error("Plug-in GPG file [%s] download failed: %s" % (digsig_url,e.strerror))
             return PLUGIN_INSTALL_ERROR_DIGITAL_SIG_NOT_FOUND, e.strerror
 
         if self.isErrorPage(file(digsig_file, 'r').read(1024)):
